import re

from pyppeteer.page import Page


async def user_agent_override(
    page: Page,
    user_agent: str = None,
    locale: str = "en-US,en",
    mask_linux: str = True,
    **kwargs
) -> None:


    ua = user_agent or (await page.browser.userAgent()).replace('HeadlessChrome', 'Chrome')

    if mask_linux and 'Linux' in ua and 'Android' not in ua: # Skip Android user agents since they also contain Linux
        ua = re.sub(r'\(([^)]+)\)', '(Windows NT 10.0; Win64; x64)', ua, 1) # Replace the first part in parentheses with Windows data

    ua_version = re.search(r'Chrome\/([\d|.]+)', ua)[1] if 'Chrome/' in ua else re.search(r'\/([\d|.]+)', (await page.browser.version()))[1]

    # Get platform identifier (short or long version)
    def _get_platform(extended=False):
      if 'Mac OS X' in ua:
        return 'Mac OS X' if extended else 'MacIntel'
      elif 'Android' in ua:
        return 'Android'
      elif 'Linux' in ua:
        return 'Linux'
      else:
        return 'Windows' if extended else 'Win32'
    
    # Source in C++: https://source.chromium.org/chromium/chromium/src/+/master:components/embedder_support/user_agent_utils.cc;l=55-100
    def _get_brands():
        seed = ua_version.split('.')[0] # the major version number of Chrome

        order = [
            [0, 1, 2],
            [0, 2, 1],
            [1, 0, 2],
            [1, 2, 0],
            [2, 0, 1],
            [2, 1, 0]
        ][int(seed) % 6]
        escaped_chars = [' ', ' ', ';']

        greasey_brand = '{}Not{}A{}Brand'.format(escaped_chars[order[0]], escaped_chars[order[1]], escaped_chars[order[2]])

        greased_brand_version_list = [None]*3
        greased_brand_version_list [order[0]] = {
            'brand': greasey_brand,
            'version': '99'
        }
        greased_brand_version_list [order[1]] = {
            'brand': 'Chromium',
            'version': seed
        }
        greased_brand_version_list [order[2]] = {
            'brand': 'Google Chrome',
            'version': seed
        }

        return greased_brand_version_list 
    
    # Return OS version
    def _get_platform_version():
      if 'Mac OS X ' in ua:
        return re.search(r'Mac OS X ([^)]+)', ua)[1]
      elif 'Android ' in ua:
        return re.search(r'Android ([^;]+)', ua)[1]
      elif 'Windows ' in ua:
        return re.search(r'Windows .*?([\d|.]+);', ua)[1]
      else:
        return ''
    
    # Get architecture, this seems to be empty on mobile and x86 on desktop
    def _get_platform_arch():
        return '' if _get_mobile() else 'x86'

    # Return the Android model, empty on desktop
    def _get_platform_model():
        return re.search(r'Android.*?;\s([^)]+)', ua)[1] if _get_mobile() else ''

    def _get_mobile():
        return 'Android' in ua
    
    override = {
      'userAgent': ua,
      'platform': _get_platform(),
      'userAgentMetadata': {
        'brands': _get_brands(),
        'fullVersion': ua_version,
        'platform': _get_platform(True),
        'platformVersion': _get_platform_version(),
        'architecture': _get_platform_arch(),
        'model': _get_platform_model(),
        'mobile': _get_mobile()
      }
    }

    # In case of headless, override the acceptLanguage in CDP.
    # This is not preferred, as it messed up the header order.
    # On headful, we set the user preference language setting instead.
    if '--headless' in page.browser.process.args:
        override['acceptLanguage'] = locale or 'en-US,en'

    await page._client.send('Network.setUserAgentOverride', override)



from pathlib import Path

from pyppeteer.page import Page


async def navigator_vendor(page: Page, vendor: str = "Google Inc.", **kwargs) -> None:
    await page.evaluateOnNewDocument(
        Path(__file__).parent.joinpath("js/navigator.vendor.js").read_text(), vendor
    )



from pyppeteer.page import Page


# https://github.com/berstend/puppeteer-extra/blob/9eacda61fc34f7994a0aa253097c473c19c0d1bf/packages/puppeteer-extra-plugin-stealth/evasions/sourceurl/index.js
async def sourceurl(page: Page, **kwargs) -> None:
    if (not page or not page._client or not page._client.send):
        return
    
    def new_send(method, params = None):
        if not method or not params:
            return page._client.old_send(method, params)

        methodsToPatch = {
            'Runtime.evaluate': 'expression',
            'Runtime.callFunctionOn': 'functionDeclaration'
        }
        SOURCE_URL_SUFFIX = '//# sourceURL=__pyppeteer_evaluation_script__'
        if method in methodsToPatch:
            params[methodsToPatch[method]] = params[methodsToPatch[method]].replace(SOURCE_URL_SUFFIX, '')
        return page._client.old_send(method, params)
            
    page._client.old_send = page._client.send
    page._client.send = new_send



from pathlib import Path

from pyppeteer.page import Page


async def chrome_runtime(page: Page, run_on_insecure_origins: bool = False, **kwargs) -> None:
    await page.evaluateOnNewDocument(
        Path(__file__).parent.joinpath("js/chrome.runtime.js").read_text(),
        run_on_insecure_origins,
    )



from pathlib import Path

from pyppeteer.page import Page


async def navigator_languages(page: Page, languages: [str] = ["en-US", "en"], **kwargs) -> None:
    await page.evaluateOnNewDocument(
        Path(__file__).parent.joinpath("js/navigator.languages.js").read_text(),
        languages,
    )



from pyppeteer.page import Page

from .chrome_app import chrome_app
from .chrome_runtime import chrome_runtime
from .iframe_content_window import iframe_content_window
from .media_codecs import media_codecs
from .sourceurl import sourceurl
from .navigator_hardware_concurrency import navigator_hardware_concurrency
from .navigator_languages import navigator_languages
from .navigator_permissions import navigator_permissions
from .navigator_plugins import navigator_plugins
from .navigator_vendor import navigator_vendor
from .navigator_webdriver import navigator_webdriver
from .user_agent_override import user_agent_override
from .utils import with_utils
from .webgl_vendor import webgl_vendor
from .window_outerdimensions import window_outerdimensions


async def stealth(page: Page, **kwargs) -> None:
    if not isinstance(page, Page):
        raise ValueError("page must be pyppeteer.page.Page")

    await with_utils(page, **kwargs)
    await chrome_app(page, **kwargs)
    await chrome_runtime(page, **kwargs)
    await iframe_content_window(page, **kwargs)
    await media_codecs(page, **kwargs)
    await sourceurl(page, **kwargs)
    await navigator_hardware_concurrency(page, **kwargs)
    await navigator_languages(page, **kwargs)
    await navigator_permissions(page, **kwargs)
    await navigator_plugins(page, **kwargs)
    await navigator_vendor(page, **kwargs)
    await navigator_webdriver(page, **kwargs)
    await user_agent_override(page, **kwargs)
    await webgl_vendor(page, **kwargs)
    await window_outerdimensions(page, **kwargs)



from pathlib import Path

from pyppeteer.page import Page


async def window_outerdimensions(page: Page, **kwargs) -> None:
    await page.evaluateOnNewDocument(
        Path(__file__).parent.joinpath("js/window.outerdimensions.js").read_text()
    )



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyppeteer_stealth",
    version="2.7.2",
    author="MeiK2333",
    author_email="meik2333@gmail.com",
    description="pyppeteer stealth",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MeiK2333/pyppeteer-stealth",
    packages=setuptools.find_packages(),
    package_data={"pyppeteer_stealth": ["js/*.js"]},
)



from pathlib import Path

from pyppeteer.page import Page


async def webgl_vendor(
    page: Page,
    webgl_vendor: str = "Intel Inc.",
    renderer: str = "Intel Iris OpenGL Engine",
    **kwargs
) -> None:
    await page.evaluateOnNewDocument(
        Path(__file__).parent.joinpath("js/webgl.vendor.js").read_text(),
        webgl_vendor,
        renderer,
    )



from pathlib import Path

from pyppeteer.page import Page


async def navigator_hardware_concurrency(page: Page, hardwareConcurrency: int = 4, **kwargs) -> None:
    await page.evaluateOnNewDocument(
        Path(__file__).parent.joinpath("js/navigator.hardwareConcurrency.js").read_text(),
        hardwareConcurrency,
    )
